@@ -2,20 +2,21 @@ package Test::TCP;
 use strict;
 use warnings;
 use 5.00800;
-our $VERSION = '1.02_02';
+our $VERSION = '1.07_01';
 use base qw/Exporter/;
 use IO::Socket::INET;
-use Test::SharedFork;
+use Test::SharedFork 0.12;
 use Test::More ();
 use Config;
 use POSIX;
 use Time::HiRes ();
+use Carp ();
+
+our @EXPORT = qw/ empty_port test_tcp wait_port /;
 
 # process does not die when received SIGTERM, on win32.
 my $TERMSIG = $^O eq 'MSWin32' ? 'KILL' : 'TERM';
 
-our @EXPORT = qw/ empty_port test_tcp wait_port /;
-
 sub empty_port {
     my $port = do {
         if (@_) {
@@ -46,42 +47,12 @@ sub test_tcp {
     for my $k (qw/client server/) {
         die "missing madatory parameter $k" unless exists $args{$k};
     }
-    my $port = $args{port} || empty_port();
-
-    if ( my $pid = Test::SharedFork->fork() ) {
-        # parent.
-        wait_port($port);
-
-        my $guard = Test::TCP::Guard->new(code => sub {
-            # cleanup
-            kill $TERMSIG => $pid;
-            local $?; # waitpid modifies original $?.
-            LOOP: while (1) {
-                my $kid = waitpid( $pid, 0 );
-                if ($^O ne 'MSWin32') { # i'm not in hell
-                    if (WIFSIGNALED($?)) {
-                        my $signame = (split(' ', $Config{sig_name}))[WTERMSIG($?)];
-                        if ($signame =~ /^(ABRT|PIPE)$/) {
-                            Test::More::diag("your server received SIG$signame");
-                        }
-                    }
-                }
-                if ($kid == 0 || $kid == -1) {
-                    last LOOP;
-                }
-            }
-        });
-
-        $args{client}->($port, $pid);
-    }
-    elsif ( $pid == 0 ) {
-        # child
-        $args{server}->($port);
-        exit;
-    }
-    else {
-        die "fork failed: $!";
-    }
+    my $server = Test::TCP->new(
+        code => $args{server},
+        port => $args{port} || empty_port(),
+    );
+    $args{client}->($server->port, $server->pid);
+    undef $server; # make sure
 }
 
 sub _check_port {
@@ -112,16 +83,64 @@ sub wait_port {
     die "cannot open port: $port";
 }
 
-{
-    package # hide from pause
-        Test::TCP::Guard;
-    sub new {
-        my ($class, %args) = @_;
-        bless { %args }, $class;
+# ------------------------------------------------------------------------- 
+# OO-ish interface
+
+sub new {
+    my $class = shift;
+    my %args = @_==1 ? %{$_[0]} : @_;
+    Carp::croak("missing mandatory parameter 'code'") unless exists $args{code};
+    my $self = bless {
+        auto_start => 1,
+        _my_pid    => $$,
+        %args,
+    }, $class;
+    $self->{port} = Test::TCP::empty_port() unless exists $self->{port};
+    $self->start()
+      if $self->{auto_start};
+    return $self;
+}
+
+sub pid  { $_[0]->{pid} }
+sub port { $_[0]->{port} }
+
+sub start {
+    my $self = shift;
+    if ( my $pid = fork() ) {
+        # parent.
+        Test::TCP::wait_port($self->port);
+        $self->{pid} = $pid;
+        return;
+    } elsif ($pid == 0) {
+        # child process
+        $self->{code}->($self->port);
+        exit 0;
+    } else {
+        die "fork failed: $!";
     }
-    sub DESTROY {
-        my ($self) = @_;
-        $self->{code}->();
+}
+
+sub DESTROY {
+    my $self = shift;
+
+    return unless defined $self->{pid};
+    return unless $self->{_my_pid} == $$;
+
+    kill $TERMSIG => $self->{pid};
+    local $?; # waitpid modifies original $?.
+    LOOP: while (1) {
+        my $kid = waitpid( $self->{pid}, 0 );
+        if ($^O ne 'MSWin32') { # i'm not in hell
+            if (POSIX::WIFSIGNALED($?)) {
+                my $signame = (split(' ', $Config{sig_name}))[POSIX::WTERMSIG($?)];
+                if ($signame =~ /^(ABRT|PIPE)$/) {
+                    Test::More::diag("your server received SIG$signame");
+                }
+            }
+        }
+        if ($kid == 0 || $kid == -1) {
+            last LOOP;
+        }
     }
 }
 
@@ -139,7 +158,7 @@ Test::TCP - testing TCP program
     use Test::TCP;
     test_tcp(
         client => sub {
-            my $port = shift;
+            my ($port, $server_pid) = @_;
             # send request to the server
         },
         server => sub {
@@ -150,6 +169,7 @@ Test::TCP - testing TCP program
 
 using other server program
 
+    use Test::TCP;
     test_tcp(
         client => sub {
             my $port = shift;
@@ -160,6 +180,19 @@ using other server program
         },
     );
 
+Or, OO-ish interface
+
+    use Test::TCP;
+
+    my $server = Test::TCP->new(
+        code => sub {
+            my $port = shift;
+            ...
+        },
+    );
+    my $client = MyClient->new(host => '127.0.0.1', port => $server->port);
+    undef $server; # kill child process on DESTROY
+
 =head1 DESCRIPTION
 
 Test::TCP is test utilities for TCP/IP program.
@@ -197,6 +230,46 @@ Waits for a particular port is available for connect.
 
 =back
 
+=head1 OO-ish interface
+
+=over 4
+
+=item my $server = Test::TCP->new(%args);
+
+Create new instance of Test::TCP.
+
+Arguments are following:
+
+=over 4
+
+=item $args{auto_start}: Boolean
+
+Call C<< $server->start() >> after create instance.
+
+Default: true
+
+=item $args{code}: CodeRef
+
+The callback function. Argument for callback function is: C<< $code->($pid) >>.
+
+This parameter is required.
+
+=back
+
+=item $server->start()
+
+Start the server process. Normally, you don't need to call this method.
+
+=item my $pid = $server->pid();
+
+Get the pid of child process.
+
+=item my $port = $server->port();
+
+Get the port number of child process.
+
+=back
+
 =head1 FAQ
 
 =over 4
@@ -225,6 +298,20 @@ You can call test_tcp() twice!
         },
     );
 
+Or use OO-ish interface instead.
+
+    my $server1 = Test::TCP->new(code => sub {
+        my $port1 = shift;
+        ...
+    });
+    my $server2 = Test::TCP->new(code => sub {
+        my $port2 = shift;
+        ...
+    });
+
+    # your client code here.
+    ...
+
 =back
 
 =head1 AUTHOR
@@ -239,6 +326,10 @@ dragon3
 
 charsbar
 
+Tatsuhiko Miyagawa
+
+lestrrat
+
 =head1 SEE ALSO
 
 =head1 LICENSE
